@@ -1,6 +1,6 @@
 /*
     pkTriggerCord
-    Copyright (C) 2011-2017 Andras Salamon <andras.salamon@melda.info>
+    Copyright (C) 2011-2018 Andras Salamon <andras.salamon@melda.info>
     Remote control of Pentax DSLR cameras.
 
     Support for K200D added by Jens Dreyer <jens.dreyer@udo.edu> 04/2011
@@ -38,11 +38,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdbool.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include "js0n.h"
 
 #include "pslr_model.h"
+#include "pslr.h"
 
 static uint8_t lastbuf[MAX_STATUS_BUF_SIZE];
 static int first = 1;
+static char *jsontext=NULL;
+static int jsonsize;
 
 static void ipslr_status_diff(uint8_t *buf) {
     int n;
@@ -66,6 +72,7 @@ static void ipslr_status_diff(uint8_t *buf) {
     }
 }
 
+static
 uint16_t get_uint16_be(uint8_t *buf) {
     uint16_t res;
     res = buf[0] << 8 | buf[1];
@@ -78,12 +85,14 @@ uint32_t get_uint32_be(uint8_t *buf) {
     return res;
 }
 
+static
 int32_t get_int32_be(uint8_t *buf) {
     int32_t res;
     res = buf[0] << 24 | buf[1] << 16 | buf[2] << 8 | buf[3];
     return res;
 }
 
+static
 uint16_t get_uint16_le(uint8_t *buf) {
     uint16_t res;
     res = buf[1] << 8 | buf[0];
@@ -96,6 +105,7 @@ uint32_t get_uint32_le(uint8_t *buf) {
     return res;
 }
 
+static
 int32_t get_int32_le(uint8_t *buf) {
     int32_t res;
     res = buf[3] << 24 | buf[2] << 16 | buf[1] << 8 | buf[0];
@@ -165,6 +175,7 @@ const char* int_to_binary( uint16_t x ) {
 }
 
 
+static
 int _get_user_jpeg_stars( ipslr_model_info_t *model, int hwqual ) {
     if ( model->id == 0x12f71 ) {
         // K5IIs hack
@@ -194,6 +205,7 @@ int get_hw_jpeg_quality( ipslr_model_info_t *model, int user_jpeg_stars) {
 }
 
 
+static
 void ipslr_status_parse_k10d(ipslr_handle_t  *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -238,6 +250,7 @@ void ipslr_status_parse_k10d(ipslr_handle_t  *p, pslr_status *status) {
     status->focus = get_int32_be(&buf[0x174]);
 }
 
+static
 void ipslr_status_parse_k20d(ipslr_handle_t *p, pslr_status *status) {
 
     uint8_t *buf = p->status_buffer;
@@ -287,6 +300,7 @@ void ipslr_status_parse_k20d(ipslr_handle_t *p, pslr_status *status) {
     // 0x160 and 0x164 change when AF
 }
 
+static
 void ipslr_status_parse_istds(ipslr_handle_t *p, pslr_status *status) {
 
     uint8_t *buf = p->status_buffer;
@@ -310,6 +324,7 @@ void ipslr_status_parse_istds(ipslr_handle_t *p, pslr_status *status) {
 // this method is used for K-x, K-7, K-5, K-r
 //
 // some cameras also have this data block, but it's shifted a bit
+static
 void ipslr_status_parse_common(ipslr_handle_t *p, pslr_status *status, int shift) {
 
     uint8_t *buf = p->status_buffer;
@@ -394,6 +409,7 @@ void ipslr_status_parse_common(ipslr_handle_t *p, pslr_status *status, int shift
 
 }
 
+static
 void ipslr_status_parse_kx(ipslr_handle_t *p, pslr_status *status) {
 
     uint8_t *buf = p->status_buffer;
@@ -412,6 +428,7 @@ void ipslr_status_parse_kx(ipslr_handle_t *p, pslr_status *status) {
 
 // Vince: K-r support 2011-06-22
 //
+static
 void ipslr_status_parse_kr(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -427,6 +444,7 @@ void ipslr_status_parse_kr(ipslr_handle_t *p, pslr_status *status) {
     status->lens_id2 = get_uint32_be( &buf[0x198]);
 }
 
+static
 void ipslr_status_parse_k5(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -445,6 +463,7 @@ void ipslr_status_parse_k5(ipslr_handle_t *p, pslr_status *status) {
 //status.focused = getInt32(statusBuf, 0x164);
 }
 
+static
 void ipslr_status_parse_k30(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -465,6 +484,7 @@ void ipslr_status_parse_k30(ipslr_handle_t *p, pslr_status *status) {
 }
 
 // status check seems to be the same as K30
+static
 void ipslr_status_parse_k01(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -484,6 +504,7 @@ void ipslr_status_parse_k01(ipslr_handle_t *p, pslr_status *status) {
     status->lens_id2 = get_uint32_be( &buf[0x19C]); // - good for K01
 }
 
+static
 void ipslr_status_parse_k50(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -499,6 +520,7 @@ void ipslr_status_parse_k50(ipslr_handle_t *p, pslr_status *status) {
     status->lens_id2 = get_uint32_be( &buf[0x19C]);
 }
 
+static
 void ipslr_status_parse_k500(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -517,6 +539,7 @@ void ipslr_status_parse_k500(ipslr_handle_t *p, pslr_status *status) {
     status->max_shutter_speed.denom = 6000;
 }
 
+static
 void ipslr_status_parse_km(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -534,6 +557,7 @@ void ipslr_status_parse_km(ipslr_handle_t *p, pslr_status *status) {
 }
 
 // K-3 returns data in little-endian
+static
 void ipslr_status_parse_k3(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -550,6 +574,7 @@ void ipslr_status_parse_k3(ipslr_handle_t *p, pslr_status *status) {
     status->lens_id2 = get_uint32_le( &buf[0x19C]);
 }
 
+static
 void ipslr_status_parse_ks1(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -567,6 +592,7 @@ void ipslr_status_parse_ks1(ipslr_handle_t *p, pslr_status *status) {
 }
 
 
+static
 void ipslr_status_parse_k1(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -606,6 +632,7 @@ void ipslr_status_parse_k1(ipslr_handle_t *p, pslr_status *status) {
     status->lens_id2 = get_uint32_le( &buf[0x1A0]);
 }
 
+static
 void ipslr_status_parse_k70(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -650,6 +677,7 @@ void ipslr_status_parse_k70(ipslr_handle_t *p, pslr_status *status) {
     status->shake_reduction = get_uint32_le(&buf[0xe4]);
 }
 
+static
 void ipslr_status_parse_k200d(ipslr_handle_t *p, pslr_status *status) {
     uint8_t *buf = p->status_buffer;
     if ( debug ) {
@@ -703,39 +731,211 @@ void ipslr_status_parse_k200d(ipslr_handle_t *p, pslr_status *status) {
     // 4= remote, 5= remote 3s delay
 }
 
+pslr_setting_def_t *find_setting_by_name (pslr_setting_def_t *array, int array_length, char *name) {
+    if (array == NULL || array_length == 0) {
+        return NULL;
+    }
+    int i;
+    for ( i = 0; i<array_length; i++) {
+        if ( strncmp(array[i].name, name, strlen(name))==0 ) {
+            return &array[i];
+        }
+    }
+    return NULL;
+}
+
+static
+char *read_json_file(int *jsonsize) {
+    int jsonfd = open("pentax_settings.json", O_RDONLY);
+    if (jsonfd == -1) {
+        // cannot find in the current directory, also checking PKTDATADIR
+        jsonfd = open(PKTDATADIR "/pentax_settings.json", O_RDONLY);
+        if (jsonfd == -1) {
+            fprintf(stderr, "Cannot open pentax_settings.json file\n");
+            return NULL;
+        }
+    }
+    *jsonsize = lseek(jsonfd, 0, SEEK_END);
+    lseek(jsonfd, 0, SEEK_SET);
+    char *jsontext=malloc(*jsonsize);
+    read(jsonfd, jsontext, *jsonsize);
+    DPRINT("json text:\n%s\n", jsontext);
+    return jsontext;
+}
+
+pslr_setting_def_t *setting_file_process(const char *cameraid, int *def_num) {
+    pslr_setting_def_t defs[128];
+    *def_num=0;
+    if (jsontext == NULL) {
+        jsontext = read_json_file(&jsonsize);
+    }
+    size_t json_part_length;
+    const char *json_part;
+    if (!(json_part = js0n(cameraid, strlen(cameraid), jsontext, jsonsize, &json_part_length))) {
+        fprintf(stderr, "JSON error\n");
+        return NULL;
+    }
+
+    if (!(json_part = js0n("fields", strlen("fields"), json_part, json_part_length, &json_part_length))) {
+        fprintf(stderr, "No fields defined for the camera\n");
+        return NULL;
+    }
+    int ai=0;
+    const char *json_array_part;
+    size_t json_array_part_length;
+    size_t name_length, type_length, value_length, address_length;
+    while ((json_array_part = js0n(NULL, ai, json_part, json_part_length, &json_array_part_length))) {
+        const char *camera_field_name_ptr;
+        char *camera_field_name;
+        if (!(camera_field_name_ptr=js0n( "name", strlen("name"), json_array_part, json_array_part_length, &name_length))) {
+            fprintf(stderr, "No name is defined\n");
+            return NULL;
+        } else {
+            camera_field_name=malloc(name_length+1);
+            memcpy(camera_field_name, camera_field_name_ptr, name_length);
+            camera_field_name[name_length]='\0';
+        }
+        const char *camera_field_type_ptr;
+        char *camera_field_type;
+        if (!(camera_field_type_ptr=js0n( "type", strlen("type"), json_array_part, json_array_part_length, &type_length))) {
+            fprintf(stderr, "No type is defined\n");
+            return NULL;
+        } else {
+            camera_field_type=malloc(type_length+1);
+            memcpy(camera_field_type, camera_field_type_ptr, type_length);
+            camera_field_type[type_length]='\0';
+        }
+        const char *camera_field_value_ptr;
+        char *camera_field_value;
+        if (!(camera_field_value_ptr=js0n( "value", strlen("value"), json_array_part, json_array_part_length, &value_length))) {
+            camera_field_value=NULL;
+        } else {
+            camera_field_value=malloc(value_length+1);
+            memcpy(camera_field_value, camera_field_value_ptr, value_length);
+            camera_field_value[value_length]='\0';
+        }
+
+        const char *camera_field_address_ptr;
+        char *camera_field_address;
+        if (!(camera_field_address_ptr=js0n( "address", strlen("address"), json_array_part, json_array_part_length, &address_length))) {
+            camera_field_address=NULL;
+        } else {
+            camera_field_address=malloc(address_length+1);
+            memcpy(camera_field_address, camera_field_address_ptr, address_length);
+            camera_field_address[address_length]='\0';
+        }
+        DPRINT("name: %.*s %.*s %.*s %.*s\n", (int)name_length, camera_field_name, (int)address_length, camera_field_address, (int)value_length, camera_field_value, (int)type_length, camera_field_type);
+        pslr_setting_def_t setting_def = { camera_field_name, camera_field_address==NULL?0:strtoul(camera_field_address,NULL,16), camera_field_value, camera_field_type };
+        defs[(*def_num)++]=setting_def;
+        ++ai;
+    }
+    pslr_setting_def_t *ret=malloc(*def_num*sizeof(pslr_setting_def_t));
+    //        printf("return %d defs\n",*def_num);
+    memcpy(ret, defs, *def_num*sizeof(pslr_setting_def_t));
+    return ret;
+}
+
+void ipslr_settings_parser_json(const char *cameraid, ipslr_handle_t *p, pslr_settings *settings) {
+    uint8_t *buf = p->settings_buffer;
+    memset(settings, 0, sizeof (*settings));
+    int def_num;
+
+    pslr_setting_def_t *defs = setting_file_process(cameraid,&def_num);
+    int def_index=0;
+    while (def_index < def_num) {
+        pslr_bool_setting bool_setting;
+        pslr_uint16_setting uint16_setting;
+        if (strncmp(defs[def_index].type, "boolean", 7)==0) {
+            if (defs[def_index].value!=NULL) {
+                bool_setting = (pslr_bool_setting) {
+                    PSLR_SETTING_STATUS_HARDWIRED, strcmp("false", defs[def_index].value)==0 ? false : true
+                };
+            } else if (defs[def_index].address!=0) {
+                uint8_t target = strcmp(defs[def_index].type, "boolean!")==0 ? 0 : 1;
+                bool_setting = (pslr_bool_setting) {
+                    PSLR_SETTING_STATUS_READ, buf[defs[def_index].address] == target
+                };
+            } else {
+                bool_setting = (pslr_bool_setting) {
+                    PSLR_SETTING_STATUS_HARDWIRED, false
+                };
+            }
+        } else if (strcmp(defs[def_index].type, "uint16")==0) {
+            if (defs[def_index].value!=NULL) {
+                uint16_setting = (pslr_uint16_setting) {
+                    PSLR_SETTING_STATUS_HARDWIRED, strcmp("false", defs[def_index].value)==0 ? false : true
+                };
+            } else if (defs[def_index].address!=0) {
+                uint16_setting = (pslr_uint16_setting) {
+                    PSLR_SETTING_STATUS_READ, get_uint16_be(&buf[defs[def_index].address])
+                };
+            } else {
+                uint16_setting = (pslr_uint16_setting) {
+                    PSLR_SETTING_STATUS_NA, 0
+                };
+            }
+        } else {
+            fprintf(stderr, "Invalid json type: %s\n", defs[def_index].type);
+        }
+        if (strcmp(defs[def_index].name, "bulb_mode_press_press")==0) {
+            settings->bulb_mode_press_press = bool_setting;
+        } else if (strcmp(defs[def_index].name, "one_push_bracketing")==0) {
+            settings->one_push_bracketing = bool_setting;
+        } else if (strcmp(defs[def_index].name, "bulb_timer")==0) {
+            settings->bulb_timer = bool_setting;
+        } else if (strcmp(defs[def_index].name, "bulb_timer_sec")==0) {
+            settings->bulb_timer_sec = uint16_setting;
+        } else if (strcmp(defs[def_index].name, "using_aperture_ring")==0) {
+            settings->using_aperture_ring = bool_setting;
+        } else if (strcmp(defs[def_index].name, "shake_reduction")==0) {
+            settings->shake_reduction = bool_setting;
+        } else if (strcmp(defs[def_index].name, "astrotracer")==0) {
+            settings->astrotracer = bool_setting;
+        } else if (strcmp(defs[def_index].name, "astrotracer_timer_sec")==0) {
+            settings->astrotracer_timer_sec = uint16_setting;
+        } else if (strcmp(defs[def_index].name, "horizon_correction")==0) {
+            settings->horizon_correction = bool_setting;
+        }
+
+        ++def_index;
+    }
+}
+
+
 ipslr_model_info_t camera_models[] = {
-    { 0x12aa2, "*ist DS",     1, 1, 0, 264, 3, {6, 4, 2},       5, 4000, 200, 3200, 200,  3200,  PSLR_JPEG_IMAGE_TONE_BRIGHT,        0, 11, ipslr_status_parse_istds },
-    { 0x12cd2, "K20D",        0, 1, 0, 412, 4, {14, 10, 6, 2},  7, 4000, 100, 3200, 100,  6400,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,    1, 11, ipslr_status_parse_k20d  },
-    { 0x12c1e, "K10D",        0, 1, 0, 392, 3, {10, 6, 2},      7, 4000, 100, 1600, 100,  1600,  PSLR_JPEG_IMAGE_TONE_BRIGHT,        0, 11, ipslr_status_parse_k10d  },
-    { 0x12c20, "GX10",        0, 1, 0, 392, 3, {10, 6, 2},      7, 4000, 100, 1600, 100,  1600,  PSLR_JPEG_IMAGE_TONE_BRIGHT,        0, 11, ipslr_status_parse_k10d  },
-    { 0x12cd4, "GX20",        0, 1, 0, 412, 4, {14, 10, 6, 2},  7, 4000, 100, 3200, 100,  6400,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,    1, 11, ipslr_status_parse_k20d  },
-    { 0x12dfe, "K-x",         0, 1, 0, 436, 3, {12, 10, 6, 2},  9, 6000, 200, 6400, 100, 12800,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,    1, 11, ipslr_status_parse_kx    }, //muted: bug
-    { 0x12cfa, "K200D",       0, 1, 0, 408, 3, {10, 6, 2},      9, 4000, 100, 1600, 100,  1600,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,    1, 11, ipslr_status_parse_k200d },
-    { 0x12db8, "K-7",         0, 1, 0, 436, 4, {14, 10, 6, 2},  9, 8000, 100, 3200, 100,  6400,  PSLR_JPEG_IMAGE_TONE_MUTED,         1, 11, ipslr_status_parse_kx    },
-    { 0x12e6c, "K-r",         0, 1, 0, 440, 3, {12, 10, 6, 2},  9, 6000, 200,12800, 100, 25600,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 11, ipslr_status_parse_kr    },
-    { 0x12e76, "K-5",         0, 1, 0, 444, 4, {16, 10, 6, 2},  9, 8000, 100,12800,  80, 51200,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 11, ipslr_status_parse_k5    },
-    { 0x12d72, "K-2000",      0, 1, 0, 412, 3, {10, 6, 2},      9, 4000, 100, 3200, 100,  3200,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,    1, 11, ipslr_status_parse_km    },
-    { 0x12d73, "K-m",         0, 1, 0, 412, 3, {10, 6, 2},      9, 4000, 100, 3200, 100,  3200,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,    1, 11, ipslr_status_parse_km    },
-    { 0x12f52, "K-30",        0, 0, 0, 452, 3, {16, 12, 8, 5},  9, 6000, 100,12800, 100, 25600,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 11, ipslr_status_parse_k30   },
-    { 0x12ef8, "K-01",        0, 1, 0, 452, 3, {16, 12, 8, 5},  9, 4000, 100,12800, 100, 25600,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 11, ipslr_status_parse_k01   },
-    { 0x12f70, "K-5II",       0, 1, 0, 444,  4, {16, 10, 6, 2}, 9, 8000, 100, 12800, 80, 51200,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 11, ipslr_status_parse_k5    },
-    { 0x12f71, "K-5IIs",      0, 1, 0, 444,  4, {16, 10, 6, 2}, 9, 8000, 100, 12800, 80, 51200,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 11, ipslr_status_parse_k5    },
-    { 0x12fb6, "K-50",        0, 1, 0, 452,  4, {16, 12, 8, 5}, 9, 6000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 11, ipslr_status_parse_k50   },
-    { 0x12fc0, "K-3"        , 0, 1, 1, 452,  4, {24, 14, 6, 2}, 9, 8000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 27, ipslr_status_parse_k3    },
-    { 0x1309c, "K-3II"      , 0, 1, 1, 452,  4, {24, 14, 6, 2}, 9, 8000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS, 1, 27, ipslr_status_parse_k3    },
-    { 0x12fca, "K-500",       0, 1, 0, 452,  3, {16, 12, 8, 5}, 9, 6000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_CROSS_PROCESSING, 1, 11, ipslr_status_parse_k500  },
+    { 0x12aa2, "*ist DS",     true,  true,  true,  false, false, false, 264, 3, {6, 4, 2},       5, 4000, 200, 3200, 200,  3200,  PSLR_JPEG_IMAGE_TONE_BRIGHT,           false, 11, ipslr_status_parse_istds},
+    { 0x12cd2, "K20D",        false, true,  true,  false, false, false, 412, 4, {14, 10, 6, 2},  7, 4000, 100, 3200, 100,  6400,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,       true,  11, ipslr_status_parse_k20d},
+    { 0x12c1e, "K10D",        false, true,  true,  false, false, false, 392, 3, {10, 6, 2},      7, 4000, 100, 1600, 100,  1600,  PSLR_JPEG_IMAGE_TONE_BRIGHT,           false, 11, ipslr_status_parse_k10d},
+    { 0x12c20, "GX10",        false, true,  true,  false, false, false, 392, 3, {10, 6, 2},      7, 4000, 100, 1600, 100,  1600,  PSLR_JPEG_IMAGE_TONE_BRIGHT,           false, 11, ipslr_status_parse_k10d},
+    { 0x12cd4, "GX20",        false, true,  true,  false, false, false, 412, 4, {14, 10, 6, 2},  7, 4000, 100, 3200, 100,  6400,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,       true,  11, ipslr_status_parse_k20d},
+    { 0x12dfe, "K-x",         false, true,  true,  false, false, false, 436, 3, {12, 10, 6, 2},  9, 6000, 200, 6400, 100, 12800,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,       true,  11, ipslr_status_parse_kx}, //muted: bug
+    { 0x12cfa, "K200D",       false, true,  true,  false, false, false, 408, 3, {10, 6, 2},      9, 4000, 100, 1600, 100,  1600,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,       true,  11, ipslr_status_parse_k200d},
+    { 0x12db8, "K-7",         false, true,  true,  false, false, false, 436, 4, {14, 10, 6, 2},  9, 8000, 100, 3200, 100,  6400,  PSLR_JPEG_IMAGE_TONE_MUTED,            true,  11, ipslr_status_parse_kx},
+    { 0x12e6c, "K-r",         false, true,  true,  false, false, false, 440, 3, {12, 10, 6, 2},  9, 6000, 200,12800, 100, 25600,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  11, ipslr_status_parse_kr},
+    { 0x12e76, "K-5",         false, true,  true,  false, false, false, 444, 4, {16, 10, 6, 2},  9, 8000, 100,12800,  80, 51200,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  11, ipslr_status_parse_k5},
+    { 0x12d72, "K-2000",      false, true,  true,  false, false, false, 412, 3, {10, 6, 2},      9, 4000, 100, 3200, 100,  3200,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,       true,  11, ipslr_status_parse_km},
+    { 0x12d73, "K-m",         false, true,  true,  false, false, false, 412, 3, {10, 6, 2},      9, 4000, 100, 3200, 100,  3200,  PSLR_JPEG_IMAGE_TONE_MONOCHROME,       true,  11, ipslr_status_parse_km},
+    { 0x12f52, "K-30",        false, true,  false, false, false, false, 452, 3, {16, 12, 8, 5},  9, 6000, 100,12800, 100, 25600,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  11, ipslr_status_parse_k30},
+    { 0x12ef8, "K-01",        false, true,  true,  false, false, false, 452, 3, {16, 12, 8, 5},  9, 4000, 100,12800, 100, 25600,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  11, ipslr_status_parse_k01},
+    { 0x12f70, "K-5II",       false, true,  true,  false, false, false, 444,  4, {16, 10, 6, 2}, 9, 8000, 100, 12800, 80, 51200,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  11, ipslr_status_parse_k5},
+    { 0x12f71, "K-5IIs",      false, true,  true,  false, false, false, 444,  4, {16, 10, 6, 2}, 9, 8000, 100, 12800, 80, 51200,  PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  11, ipslr_status_parse_k5},
+    { 0x12fb6, "K-50",        false, true,  true,  false, false, false, 452,  4, {16, 12, 8, 5}, 9, 6000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  11, ipslr_status_parse_k50},
+    { 0x12fc0, "K-3",         false, true,  true,  false, false, true,  452,  4, {24, 14, 6, 2}, 9, 8000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  27, ipslr_status_parse_k3},
+    { 0x1309c, "K-3II",       false, false, true,  true,  false, true,  452,  4, {24, 14, 6, 2}, 9, 8000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_BLEACH_BYPASS,    true,  27, ipslr_status_parse_k3},
+    { 0x12fca, "K-500",       false, true,  true,  false, false, false, 452,  3, {16, 12, 8, 5}, 9, 6000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_CROSS_PROCESSING, true,  11, ipslr_status_parse_k500},
     // only limited support from here
-    { 0x12994, "*ist D",      1, 1, 0, 0,   3, {6, 4, 2}, 3, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_NONE  , 0, 11, NULL}, // buffersize: 264
-    { 0x12b60, "*ist DS2",    1, 1, 0, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, 0, 11, NULL},
-    { 0x12b1a, "*ist DL",     1, 1, 0, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, 0, 11, NULL},
-    { 0x12b80, "GX-1L",       1, 1, 0, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, 0, 11, NULL},
-    { 0x12b9d, "K110D",       0, 1, 0, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, 0, 11, NULL},
-    { 0x12b9c, "K100D",       1, 1, 0, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, 0, 11, NULL},
-    { 0x12ba2, "K100D Super", 1, 1, 0, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, 0, 11, NULL},
-    { 0x1301a, "K-S1",        0, 1, 1, 452,  3, {20, 12, 6, 2}, 9, 6000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_CROSS_PROCESSING, 1, 11, ipslr_status_parse_ks1   },
-    { 0x13024, "K-S2",        0, 1, 1, 452,  3, {20, 12, 6, 2}, 9, 6000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_CROSS_PROCESSING, 1, 11, ipslr_status_parse_k3    },
-    { 0x13092, "K-1",         0, 1, 1, 456,  3, {36, 22, 12, 2}, 9, 8000, 100, 204800, 100, 204800, PSLR_JPEG_IMAGE_TONE_FLAT, 1, 33, ipslr_status_parse_k1      },
-    { 0x13222, "K-70",        0, 1, 1, 456,  3, {24, 14, 6, 2}, 9, 6000, 100, 102400, 100, 102400, PSLR_JPEG_IMAGE_TONE_AUTO, 1, 11, ipslr_status_parse_k70 }
+    { 0x12994, "*ist D",      true,  true,  true,  false, false, false, 0,   3, {6, 4, 2}, 3, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_NONE  , false, 11, NULL}, // buffersize: 264
+    { 0x12b60, "*ist DS2",    true,  true,  true,  false, false, false, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, false, 11, NULL},
+    { 0x12b1a, "*ist DL",     true,  true,  true,  false, false, false, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, false, 11, NULL},
+    { 0x12b80, "GX-1L",       true,  true,  true,  false, false, false, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, false, 11, NULL},
+    { 0x12b9d, "K110D",       false, true,  true,  false, false, false, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, false, 11, NULL},
+    { 0x12b9c, "K100D",       true,  true,  true,  false, false, false, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, false, 11, NULL},
+    { 0x12ba2, "K100D Super", true,  true,  true,  false, false, false, 0,   3, {6, 4, 2}, 5, 4000, 200, 3200, 200, 3200, PSLR_JPEG_IMAGE_TONE_BRIGHT, false, 11, NULL},
+    { 0x1301a, "K-S1",        false, true,  true,  false, false, true,  452,  3, {20, 12, 6, 2}, 9, 6000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_CROSS_PROCESSING, true,  11, ipslr_status_parse_ks1},
+    { 0x13024, "K-S2",        false, true,  true,  false, false, true,  452,  3, {20, 12, 6, 2}, 9, 6000, 100, 51200, 100, 51200, PSLR_JPEG_IMAGE_TONE_CROSS_PROCESSING, true,  11, ipslr_status_parse_k3},
+    { 0x13092, "K-1",         false, false, true,  false, false, true,  456,  3, {36, 22, 12, 2}, 9, 8000, 100, 204800, 100, 204800, PSLR_JPEG_IMAGE_TONE_FLAT, true,  33, ipslr_status_parse_k1 },
+    { 0x13222, "K-70",        false, false, true,  true,  true,  true,  456,  3, {24, 14, 6, 2}, 9, 6000, 100, 102400, 100, 102400, PSLR_JPEG_IMAGE_TONE_AUTO, true,  11, ipslr_status_parse_k70}
+
 };
 
 ipslr_model_info_t *find_model_by_id( uint32_t id ) {
